@@ -79,41 +79,42 @@ def depth_from_flow_and_cam_mats(flow, full_mats, inv_full_mats=None, camera_cen
                                       batch_shape, image_dims)[..., -1:]
 
 
-def flow_from_depth_and_cam_mats(pixel_coords1, cam1to2_full_mat, batch_shape=None, image_dims=None):
+def flow_from_depth_and_cam_mats(pixel_coords1, cam1to2_full_mat, batch_shape=None, image_shape=None):
     """
-    Compute optical flow :math:`\mathbf{U}_{1→2}\in\mathbb{R}^{h×w×2}` from frame 1 to 2, using depth map
-    :math:`\mathbf{X}\in\mathbb{R}^{h×w×1}` in frame 1, and the camera geometry.\n
+    Compute optical flow :math:`\mathbf{U}_{1→2}\in\mathbb{R}^{is×2}` from frame 1 to 2, using depth map
+    :math:`\mathbf{X}\in\mathbb{R}^{is×1}` in frame 1, and the camera geometry.\n
 
-    :param pixel_coords1: Depth scaled homogeneous pixel co-ordinates image in frame 1 *[batch_shape,h,w,3]*
+    :param pixel_coords1: Depth scaled homogeneous pixel co-ordinates image in frame 1 *[batch_shape,image_shape,3]*
     :type pixel_coords1: array
     :param cam1to2_full_mat: Camera1-to-camera2 full projection matrix *[batch_shape,3,4]*
     :type cam1to2_full_mat: array
     :param batch_shape: Shape of batch. Inferred from inputs if None.
     :type batch_shape: sequence of ints, optional
-    :param image_dims: Image dimensions. Inferred from inputs in None.
-    :type image_dims: sequence of ints, optional
-    :return: Optical flow from frame 1 to 2 *[batch_shape,h,w,2]*
+    :param image_shape: Image dimensions. Inferred from inputs in None.
+    :type image_shape: sequence of ints, optional
+    :return: Optical flow from frame 1 to 2 *[batch_shape,image_shape,2]*
     """
 
     if batch_shape is None:
-        batch_shape = pixel_coords1.shape[:-3]
+        batch_shape = cam1to2_full_mat.shape[:-2]
+    num_batch_dims = len(batch_shape)
 
-    if image_dims is None:
-        image_dims = pixel_coords1.shape[-3:-1]
+    if image_shape is None:
+        image_shape = pixel_coords1.shape[num_batch_dims:-1]
 
     # shapes as list
     batch_shape = list(batch_shape)
-    image_dims = list(image_dims)
+    image_shape = list(image_shape)
 
-    # BS x H x W x 3
+    # BS x IS x 3
     projected_pixel_coords = _ivy_tvg.ds_pixel_to_ds_pixel_coords(pixel_coords1, cam1to2_full_mat, batch_shape,
-                                                                  image_dims)
+                                                                  image_shape)
     projected_pixel_coords_normalized = projected_pixel_coords / (projected_pixel_coords[..., -1:] + MIN_DENOMINATOR)
 
-    # BS x H x W x 3
+    # BS x IS x 3
     pixel_coords1_normalized = pixel_coords1 / (pixel_coords1[..., -1:] + MIN_DENOMINATOR)
 
-    # BS x H x W x 2
+    # BS x IS x 2
     return projected_pixel_coords_normalized[..., 0:2] - pixel_coords1_normalized[..., 0:2]
 
 
@@ -320,11 +321,11 @@ def velocity_from_flow_cam_coords_and_cam_mats(flow_t_to_tm1, cam_coords_t, cam_
 
 
 def project_cam_coords_with_object_transformations(cam_coords_1, id_image, obj_ids, obj_trans,
-                                                   cam_1_to_2_ext_mat, batch_shape=None, image_dims=None):
+                                                   cam_1_to_2_ext_mat, batch_shape=None, image_shape=None):
     """
     Compute velocity image from co-ordinate image, id image, and object transformations.
 
-    :param cam_coords_1: Camera-centric homogeneous co-ordinates image in frame t *[batch_shape,h,w,4]*
+    :param cam_coords_1: Camera-centric homogeneous co-ordinates image in frame t *[batch_shape,image_shape,4]*
     :type cam_coords_1: array
     :param id_image: Image containing per-pixel object ids *[batch_shape,h,w,1]*
     :type id_image: array
@@ -336,20 +337,22 @@ def project_cam_coords_with_object_transformations(cam_coords_1, id_image, obj_i
     :type cam_1_to_2_ext_mat: array
     :param batch_shape: Shape of batch. Inferred from inputs if None.
     :type batch_shape: sequence of ints, optional
-    :param image_dims: Image dimensions. Inferred from inputs in None.
-    :type image_dims: sequence of ints, optional
-    :return: Relative velocity image *[batch_shape,h,w,3]*
+    :param image_shape: Image dimensions. Inferred from inputs in None.
+    :type image_shape: sequence of ints, optional
+    :return: Relative velocity image *[batch_shape,image_shape,3]*
     """
 
     if batch_shape is None:
-        batch_shape = cam_coords_1.shape[:-3]
+        batch_shape = cam_1_to_2_ext_mat.shape[:-2]
+    num_batch_dims = len(batch_shape)
 
-    if image_dims is None:
-        image_dims = cam_coords_1.shape[-3:-1]
+    if image_shape is None:
+        image_shape = cam_coords_1.shape[num_batch_dims:-1]
+    num_image_dims = len(image_shape)
 
     # shapes as list
     batch_shape = list(batch_shape)
-    image_dims = list(image_dims)
+    image_shape = list(image_shape)
     num_batch_dims = len(batch_shape)
 
     # Transform the co-ordinate image by each transformation
@@ -357,69 +360,70 @@ def project_cam_coords_with_object_transformations(cam_coords_1, id_image, obj_i
     # BS x (num_obj x 3) x 4
     obj_trans = _ivy.reshape(obj_trans, batch_shape + [-1, 4])
 
-    # BS x 4 x H x W
-    cam_coords_1_ = _ivy.transpose(cam_coords_1, list(range(num_batch_dims)) + [i + num_batch_dims for i in [2, 0, 1]])
+    # BS x 4 x IS
+    cam_coords_1_ = _ivy.transpose(
+        cam_coords_1, list(range(num_batch_dims)) + [i + num_batch_dims
+                                                     for i in ([num_image_dims] + list(range(num_image_dims)))])
 
-    # BS x 4 x (HxW)
+    # BS x 4 x (IS)
     cam_coords_1_ = _ivy.reshape(cam_coords_1_, batch_shape + [4, -1])
 
-    # BS x (num_obj x 3) x (HxW)
+    # BS x (num_obj x 3) x (IS)
     cam_coords_2_all_obj_trans = _ivy.matmul(obj_trans, cam_coords_1_)
 
-    # BS x (HxW) x (num_obj x 3)
-    cam_coords_2_all_obj_trans = \
-        _ivy.transpose(cam_coords_2_all_obj_trans, list(range(num_batch_dims)) + [i + num_batch_dims for i in [1, 0]])
+    # BS x (IS) x (num_obj x 3)
+    cam_coords_2_all_obj_trans = _ivy.swapaxes(cam_coords_2_all_obj_trans, -1, -2)
 
-    # BS x H x W x num_obj x 3
-    cam_coords_2_all_obj_trans = _ivy.reshape(cam_coords_2_all_obj_trans, batch_shape + image_dims + [-1, 3])
+    # BS x IS x num_obj x 3
+    cam_coords_2_all_obj_trans = _ivy.reshape(cam_coords_2_all_obj_trans, batch_shape + image_shape + [-1, 3])
 
     # Multiplier
 
-    # BS x 1 x 1 x num_obj
-    obj_ids = _ivy.reshape(obj_ids, batch_shape + [1, 1] + [-1])
+    # BS x [1]*NID x num_obj
+    obj_ids = _ivy.reshape(obj_ids, batch_shape + [1]*num_image_dims + [-1])
 
-    # BS x H x W x num_obj x 1
+    # BS x IS x num_obj x 1
     multiplier = _ivy.cast(_ivy.expand_dims(obj_ids == id_image, -1), 'float32')
 
     # compute validity mask, for pixels which are on moving objects
 
-    # BS x H x W x 1
+    # BS x IS x 1
     motion_mask = _ivy.reduce_sum(multiplier, -2) > 0
 
     # make invalid transformations equal to zero
 
-    # BS x H x W x num_obj x 3
+    # BS x IS x num_obj x 3
     cam_coords_2_all_obj_trans_w_zeros = cam_coords_2_all_obj_trans * multiplier
 
     # reduce to get only valid transformations
 
-    # BS x H x W x 3
+    # BS x IS x 3
     cam_coords_2_all_obj_trans = _ivy.reduce_sum(cam_coords_2_all_obj_trans_w_zeros, -2)
 
     # find cam coords to for zero motion pixels
 
-    # BS x H x W x 3
-    cam_coords_2_wo_motion = _ivy_tvg.cam_to_cam_coords(cam_coords_1, cam_1_to_2_ext_mat, batch_shape, image_dims)
+    # BS x IS x 3
+    cam_coords_2_wo_motion = _ivy_tvg.cam_to_cam_coords(cam_coords_1, cam_1_to_2_ext_mat, batch_shape, image_shape)
 
-    # BS x H x W x 4
+    # BS x IS x 4
     cam_coords_2_all_trans_homo =\
-        _ivy_mech.make_coordinates_homogeneous(cam_coords_2_all_obj_trans, batch_shape + image_dims)
+        _ivy_mech.make_coordinates_homogeneous(cam_coords_2_all_obj_trans, batch_shape + image_shape)
     cam_coords_2 = _ivy.where(motion_mask, cam_coords_2_all_trans_homo, cam_coords_2_wo_motion)
 
     # return
 
-    # BS x H x W x 3,    BS x H x W x 1
+    # BS x IS x 3,    BS x IS x 1
     return cam_coords_2, motion_mask
 
 
 def velocity_from_cam_coords_id_image_and_object_trans(cam_coords_t, id_image, obj_ids, obj_trans, delta_t,
-                                                       batch_shape=None, image_dims=None, dev_str=None):
+                                                       batch_shape=None, image_shape=None, dev_str=None):
     """
     Compute velocity image from co-ordinate image, id image, and object transformations.
 
-    :param cam_coords_t: Camera-centric homogeneous co-ordinates image in frame t *[batch_shape,h,w,4]*
+    :param cam_coords_t: Camera-centric homogeneous co-ordinates image in frame t *[batch_shape,image_shape,4]*
     :type cam_coords_t: array
-    :param id_image: Image containing per-pixel object ids *[batch_shape,h,w,1]*
+    :param id_image: Image containing per-pixel object ids *[batch_shape,image_shape,1]*
     :type id_image: array
     :param obj_ids: Object ids *[batch_shape,num_obj,1]*
     :type obj_ids: array
@@ -429,58 +433,59 @@ def velocity_from_cam_coords_id_image_and_object_trans(cam_coords_t, id_image, o
     :type delta_t: array
     :param batch_shape: Shape of batch. Inferred from inputs if None.
     :type batch_shape: sequence of ints, optional
-    :param image_dims: Image dimensions. Inferred from inputs in None.
-    :type image_dims: sequence of ints, optional
+    :param image_shape: Image dimensions. Inferred from inputs in None.
+    :type image_shape: sequence of ints, optional
     :param dev_str: device on which to create the array 'cuda:0', 'cuda:1', 'cpu' etc. Same as x if None.
     :type dev_str: str, optional
-    :return: Relative velocity image *[batch_shape,h,w,3]*
+    :return: Relative velocity image *[batch_shape,image_shape,3]*
     """
 
     if batch_shape is None:
-        batch_shape = cam_coords_t.shape[:-3]
+        batch_shape = delta_t.shape[:-1]
+    num_batch_dims = len(batch_shape)
 
-    if image_dims is None:
-        image_dims = cam_coords_t.shape[-3:-1]
+    if image_shape is None:
+        image_shape = cam_coords_t.shape[num_batch_dims:-1]
 
     if dev_str is None:
         dev_str = _ivy.dev_str(cam_coords_t)
 
     # shapes as list
     batch_shape = list(batch_shape)
-    image_dims = list(image_dims)
+    image_shape = list(image_shape)
 
     # get co-ordinate re-projections
 
-    # BS x H x W x 4
+    # BS x IS x 4
     cam_coords_t_all_trans, motion_mask =\
         project_cam_coords_with_object_transformations(cam_coords_t, id_image, obj_ids, obj_trans,
                                                        _ivy.identity(4, batch_shape=batch_shape)[..., 0:3, :],
-                                                       batch_shape, image_dims)
+                                                       batch_shape, image_shape)
 
-    # BS x H x W x 4
+    # BS x IS x 4
     cam_coords_t_all_trans = \
         _ivy.where(motion_mask, cam_coords_t_all_trans, _ivy.zeros_like(cam_coords_t_all_trans, dev_str=dev_str))
 
     # compute velocities
 
-    # BS x H x W x 3
+    # BS x IS x 3
     vel = (cam_coords_t[..., 0:3] - cam_coords_t_all_trans[..., 0:3])/delta_t
 
     # prune velocities
 
-    # BS x H x W x 3
+    # BS x IS x 3
     return _ivy.where(motion_mask, vel, _ivy.zeros_like(vel, dev_str=dev_str))
 
 
 def flow_from_cam_coords_id_image_and_object_trans(cam_coords_f1, id_image, obj_ids, obj_trans,
                                                    calib_mat, cam_1_to_2_ext_mat, batch_shape=None,
-                                                   image_dims=None):
+                                                   image_shape=None):
     """
     Compute optical flow from co-ordinate image, id image, and object transformations.
 
-    :param cam_coords_f1: Camera-centric homogeneous co-ordinates image in frame t *[batch_shape,h,w,4]*
+    :param cam_coords_f1: Camera-centric homogeneous co-ordinates image in frame t *[batch_shape,image_shape,4]*
     :type cam_coords_f1: array
-    :param id_image: Image containing per-pixel object ids *[batch_shape,h,w,1]*
+    :param id_image: Image containing per-pixel object ids *[batch_shape,image_shape,1]*
     :type id_image: array
     :param obj_ids: Object ids *[batch_shape,num_obj,1]*
     :type obj_ids: array
@@ -492,33 +497,34 @@ def flow_from_cam_coords_id_image_and_object_trans(cam_coords_f1, id_image, obj_
     :type cam_1_to_2_ext_mat: array
     :param batch_shape: Shape of batch. Inferred from inputs if None.
     :type batch_shape: sequence of ints, optional
-    :param image_dims: Image dimensions. Inferred from inputs in None.
-    :type image_dims: sequence of ints, optional
-    :return: Relative velocity image *[batch_shape,h,w,3]*
+    :param image_shape: Image dimensions. Inferred from inputs in None.
+    :type image_shape: sequence of ints, optional
+    :return: Relative velocity image *[batch_shape,image_shape,3]*
     """
 
     if batch_shape is None:
-        batch_shape = cam_coords_f1.shape[:-3]
+        batch_shape = calib_mat.shape[:-2]
+    num_batch_dims = len(batch_shape)
 
-    if image_dims is None:
-        image_dims = cam_coords_f1.shape[-3:-1]
+    if image_shape is None:
+        image_shape = cam_coords_f1.shape[num_batch_dims:-1]
 
     # shapes as list
     batch_shape = list(batch_shape)
-    image_dims = list(image_dims)
+    image_shape = list(image_shape)
 
     # get co-ordinate re-projections
 
-    # BS x H x W x 3
+    # BS x IS x 3
     cam_coords_trans_f2, _ =\
         project_cam_coords_with_object_transformations(cam_coords_f1, id_image, obj_ids, obj_trans,
-                                                       cam_1_to_2_ext_mat, batch_shape, image_dims)
+                                                       cam_1_to_2_ext_mat, batch_shape, image_shape)
 
     # co-ordinates to pixel co-ordinates
 
-    # BS x H x W x 3
-    pixel_coords_f1 = _ivy_svg.cam_to_ds_pixel_coords(cam_coords_f1, calib_mat, batch_shape, image_dims)
-    pixel_coords_trans_f2 = _ivy_svg.cam_to_ds_pixel_coords(cam_coords_trans_f2, calib_mat, batch_shape, image_dims)
+    # BS x IS x 3
+    pixel_coords_f1 = _ivy_svg.cam_to_ds_pixel_coords(cam_coords_f1, calib_mat, batch_shape, image_shape)
+    pixel_coords_trans_f2 = _ivy_svg.cam_to_ds_pixel_coords(cam_coords_trans_f2, calib_mat, batch_shape, image_shape)
 
     # unscaled pixel coords
     unscaled_pixel_coords_f1 = pixel_coords_f1[..., 0:2] / (pixel_coords_f1[..., -1:] + MIN_DENOMINATOR)
@@ -526,5 +532,5 @@ def flow_from_cam_coords_id_image_and_object_trans(cam_coords_f1, id_image, obj_
 
     # optical flow
 
-    # BS x H x W x 2
+    # BS x IS x 2
     return unscaled_pixel_coords_f2 - unscaled_pixel_coords_f1
